using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System.Security.Cryptography;

namespace WebITS
{
    public class Program
    {
        private static readonly string APPID = "********";
        private static readonly string API_KEY = "********************************";
        private static readonly string API_SECRET = "********************************";

        private static readonly string WebITS_URL = "http://its-api-sg.xf-yun.com/v2/its";
        private static readonly string FROM = "**";
        private static readonly string TO = "**";
        private static readonly string TEXT = "********************************";

        static void Main(string[] args)
        {
            string body = BuildHttpBody();
            var header = BuildHttpHeader(body);
            var resultMap = DoPost2(WebITS_URL, header, body);
            if (resultMap != null)
            {
                string resultStr = resultMap.ContainsKey("body") ? resultMap["body"].ToString() : string.Empty;
                Console.WriteLine("【its result】" + resultStr);

                JObject resultData = JsonConvert.DeserializeObject<JObject>(resultStr);
                int code = int.Parse(resultData["code"].ToString());
                if (code != 0)
                {
                    Console.WriteLine("Please follow the error message tips to handle");
                }
            }
            else
            {
                Console.WriteLine("Please follow the error message tips to handle");
            }
        }

        public static IDictionary<string, string> BuildHttpHeader(string body)
        {
            IDictionary<string, string> header = new Dictionary<string, string>();
            Uri url = new Uri(WebITS_URL);
            string date = GetTimestamp();
            string digestBase64 = "SHA-256=" + SignBody(body);

            string builder = "host: its-api-sg.xf-yun.com\n" +
            $"date: {date}\n" +
            "POST /v2/its HTTP/1.1\n" +
            $"digest: {digestBase64}";
            Console.WriteLine("builder:::" + builder);
            string sha = HmacSign(API_SECRET, builder);
            string authorization = $"api_key=\"{API_KEY}\", algorithm=\"hmac-sha256\", headers=\"host date request-line digest\", signature=\"{sha}\"";
            header.Add("Authorization", authorization);
            header.Add("Content-Type", "application/json");
            header.Add("Accept", "application/json,version=1.0");
            header.Add("Host", url.Host);
            header.Add("Date", date);
            header.Add("Digest", digestBase64);
            return header;
        }

        public static string BuildHttpBody()
        {
            JObject body = new JObject();
            JObject business = new JObject();
            JObject common = new JObject();
            JObject data = new JObject();

            common.Add("app_id", APPID);
            business.Add("from", FROM);
            business.Add("to", TO);

            data.Add("text", Convert.ToBase64String(Encoding.UTF8.GetBytes(TEXT)));

            body.Add("common", common);
            body.Add("business", business);
            body.Add("data", data);

            return body.ToString();
        }


        public static string SignBody(string body)
        {
            using (SHA256 sha256 = SHA256.Create())
            {
                byte[] inputBytes = Encoding.UTF8.GetBytes(body);
                byte[] hashBytes = sha256.ComputeHash(inputBytes);

                string hashString = BitConverter.ToString(hashBytes).Replace("-", "").ToLower();

                string text = Convert.ToBase64String(Encoding.UTF8.GetBytes(hashString));
                return text;
            }
        }

        public static string HmacSign(string apiSecret, string input)
        {
            Encoding encoding = Encoding.UTF8;
            using (HMACSHA256 hmac = new HMACSHA256(encoding.GetBytes(apiSecret)))
            {
                byte[] encodedData = hmac.ComputeHash(encoding.GetBytes(input));

                return Convert.ToBase64String(encodedData);
            }
        }

        public static IDictionary<string, object> DoPost2(string url, IDictionary<string, string> header, string body)
        {
            IDictionary<string, object> resultMap = new Dictionary<string, object>();
            HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create(url);
            httpWebRequest.Method = "POST";
            httpWebRequest.KeepAlive = true;
            httpWebRequest.ContentType = "application/json";
            httpWebRequest.Headers.Add("Accept", "application/json,version=1.0");
            httpWebRequest.Headers.Add("Host", header["Host"]);
            httpWebRequest.Headers.Add("Date", header["Date"]);
            httpWebRequest.Headers.Add("Digest", header["Digest"]);
            httpWebRequest.Headers.Add("Authorization", header["Authorization"]);

            using (var streamWriter = new StreamWriter(httpWebRequest.GetRequestStream()))
            {
                streamWriter.Write(body);
                streamWriter.Flush();
                streamWriter.Close();
            }

            HttpWebResponse httpWebResponse = (HttpWebResponse)httpWebRequest.GetResponse();
            if (httpWebResponse.StatusCode == HttpStatusCode.OK)
            {
                using (var streamReader = new StreamReader(httpWebResponse.GetResponseStream()))
                {
                    string result = streamReader.ReadToEnd();
                    resultMap.Add("body", result);
                }
            }
            else
            {
                using (var streamReader = new StreamReader(httpWebResponse.GetResponseStream()))
                {
                    string errorMessage = streamReader.ReadToEnd();
                    Console.WriteLine($"failed，code：{(int)httpWebResponse.StatusCode}，message：{errorMessage}");
                }
            }

            return resultMap;
        }

        private static string GetTimestamp()
        {
            DateTime now = DateTime.UtcNow;
            TimeZoneInfo gmtTimeZone = TimeZoneInfo.FindSystemTimeZoneById("UTC");
            DateTime gmtTime = TimeZoneInfo.ConvertTimeFromUtc(now, gmtTimeZone);
            string time_str = gmtTime.ToString("ddd, dd MMM yyyy HH:mm:ss zzz", System.Globalization.CultureInfo.InvariantCulture);
            return time_str;
        }

    }
}